import { loadOpenapi } from '../openapi-loader/openapi-loader.js';
import { disableLinter, openapiTypesModule } from '../utils/consts.js';
import { fastifyImports, handlersInterfacePostfix, handlersInterfacePrefix, methods, openapiOperationsImport, utilityTypesDefinition, } from './handlers-generator.constants.js';
export async function generateHandlers(openapiFilePath) {
    const openapi = await loadOpenapi(openapiFilePath);
    const handlersProperties = getHandlersProperties(openapi);
    const operations = handlersProperties.map(({ operationId, summary }) => {
        const routeString = getRouteGenericString(operationId);
        const handler = `${operationId}: ${routeString}`;
        const documentation = createDocumentation(summary);
        return `${documentation}\n\t${handler}`;
    });
    return formatHandlers(operations);
}
function createDocumentation(summary) {
    if (typeof summary == 'string') {
        return `
  /**
   * ${summary}
   */`;
    }
    return '';
}
function getRouteGeneric(operationId) {
    const routeGenricDefinition = `
            Params: Params<'${operationId}'>
            Querystring: Querystring<'${operationId}'>
            Body: Body<'${operationId}'>
            Reply: Reply<'${operationId}'>
            Headers: Headers<'${operationId}'>
        `;
    return `{\n${routeGenricDefinition}\n\t}`;
}
function getRouteGenericString(operationId) {
    return `RouteHandlerMethod<
    RawServerDefault,
    RawRequestDefaultExpression<RawServerDefault>,
    RawReplyDefaultExpression<RawServerDefault>,
    ${getRouteGeneric(operationId)}
  >`;
}
function getHandlersProperties(openapi) {
    return Object.values(openapi.paths ?? {}).flatMap(path => methods.flatMap(method => {
        if (path && path[method]) {
            const summary = path[method]?.summary;
            const operationId = path[method]?.operationId;
            if (operationId)
                return { operationId, summary };
        }
        return [];
    }));
}
function formatHandlers(operations) {
    const operationsFormatted = operations.join('\n');
    return `${disableLinter}

${fastifyImports}
${openapiOperationsImport(openapiTypesModule)}

${utilityTypesDefinition}


${handlersInterfacePrefix}
${operationsFormatted}
${handlersInterfacePostfix}
`;
}
